@@ -34,6 +34,7 @@ class InternalAutoSolve:
         self.client_key = params["client_key"]
         self.debug = params["debug"]
         self.should_alert_on_cancel = params["should_alert_on_cancel"]
+        self.queue = []
 
         self.routing_key = ""
         self.send_routing_key = ""
@@ -55,12 +56,11 @@ class InternalAutoSolve:
         self.ready = False
         self.connected = False
 
-        self.message_backlog = []
-
     def initialized(self):
         while self.attempts < self.autosolve_constants.MAX_ATTEMPTS:
             time.sleep(.5)
             if self.ready:
+                time.sleep(.5)
                 break
         return self.ready
 
@@ -145,8 +145,8 @@ class InternalAutoSolve:
             if self.connected:
 
                 self.log("Beginning message consumption", self.autosolve_constants.SUCCESS)
-                self.ready = True
                 self.delay = 0
+                self.ready = True
                 self.channel.start_consuming()
                 self.log("Stopped Message Consuming", self.autosolve_constants.SUCCESS)
                 self.emitter.emit("AutoSolveError", AutoSolveConnectionError.CONNECTION_CLOSED_UNEXPECTEDLY)
@@ -240,26 +240,38 @@ class InternalAutoSolve:
             self.log("Connection attempts unsuccessful", self.autosolve_constants.ERROR)
             self.emitter.emit("AutoSolveError", AutoSolveConnectionError.CONNECTION_REESTABLISH_FAILED)
 
-    def send(self, message, route, exchange):
-        message["createdAt"] = round(time.time())
-        message["apiKey"] = self.api_key
-        message_string = json.dumps(message)
-        byte_string = message_string.encode()
-        if self.fanoutExchangeName == exchange:
-            return self.cancel_channel.basic_publish(exchange=exchange,
-                                            routing_key=route,
-                                            body=byte_string)
-        return self.channel.basic_publish(exchange=exchange,
-                                          routing_key=route,
-                                          body=byte_string)
+    def send(self):
+        req = None
+        try:
+            req = self.queue.pop()
+            if req is not None:
+                message = req["message"]
+                route = req["route"]
+                exchange = req["exchange"]
+                message["createdAt"] = round(time.time())
+                message["apiKey"] = self.api_key
+                message_string = json.dumps(message)
+                byte_string = message_string.encode()
+                if self.fanoutExchangeName == exchange:
+                    return self.cancel_channel.basic_publish(exchange=exchange,
+                                                    routing_key=route,
+                                                    body=byte_string)
+
+                return self.channel.basic_publish(exchange=exchange,
+                                                  routing_key=route,
+                                                  body=byte_string)
+        except Exception as e:
+            print(e)
+            if req is not None:
+                self.queue.append(req)
 
     def send_token_request(self, message):
         self.log("Sending request message for task: " + str(message['taskId']), self.autosolve_constants.STATUS)
         try:
-            send_result = self.send(message, self.token_send_routing_key,
-                                    self.directExchangeName)
+            self.queue.append({"message": message, "route": self.token_send_routing_key, "exchange": self.directExchangeName})
+            send_result = self.connection.add_callback_threadsafe(self.send)
             if send_result is None:
-                self.log("Message with TaskId: " + message['taskId'] + " sent successfully",
+                self.log("Message with TaskId: " + str(message['taskId']) + " sent successfully",
                          self.autosolve_constants.SUCCESS)
         except Exception as e:
             self.log(e, self.autosolve_constants.WARNING)
@@ -272,8 +284,8 @@ class InternalAutoSolve:
 
         self.log("Sending cancel message for task: " + message['taskId'], self.autosolve_constants.STATUS)
         try:
-            send_result = self.send(message, self.cancel_send_routing_key,
-                                    self.fanoutExchangeName)
+            self.queue.append({"message": message, "route": self.cancel_send_routing_key, "exchange": self.fanoutExchangeName})
+            send_result = self.connection.add_callback_threadsafe(self.send)
             if send_result is None:
                 self.log("Cancel message with TaskId: " + message['taskId'] + " sent successfully",
                          self.autosolve_constants.SUCCESS)
@@ -285,8 +297,9 @@ class InternalAutoSolve:
         message = {"apiKey": self.api_key}
         self.log("Sending cancel all request for api key ::  " + message['apiKey'], self.autosolve_constants.STATUS)
         try:
-            send_result = self.send(message, self.cancel_send_routing_key,
-                                    self.fanoutExchangeName)
+            self.queue.append(
+                {"message": message, "route": self.cancel_send_routing_key, "exchange": self.fanoutExchangeName})
+            send_result = self.connection.add_callback_threadsafe(self.send)
             if send_result is None:
                 self.log("Cancel all requests sent successfully", self.autosolve_constants.SUCCESS)
         except Exception as e:
@@ -295,7 +308,7 @@ class InternalAutoSolve:
 
     def resend(self, message, route, exchange):
         time.sleep(5)
-        result = self.send(message, route, exchange)
+        result = self.connection.add_callback_threadsafe(self.send)
         if result is None:
             self.log("Resend attempt successful", self.autosolve_constants.SUCCESS)
         else:
@@ -304,15 +317,15 @@ class InternalAutoSolve:
             self.add_message_to_backlog({"message": message, "route": route, "exchange": exchange})
 
     def add_message_to_backlog(self, message):
-        self.message_backlog.append(message)
+        self.queue.append(message)
 
     def attempt_message_backlog_send(self):
         self.log("Sending messages from backlog", self.autosolve_constants.STATUS)
-        message_cache = self.message_backlog
-        for message in message_cache:
-            result = self.send(message["message"], message["route"], message["exchange"])
-            if result is None:
-                self.message_backlog.remove(message)
+        for message in self.queue:
+            self.log("Backlog message sending: " + str(message["message"]["taskId"]), self.autosolve_constants.STATUS)
+            result = self.connection.add_callback_threadsafe(self.send)
+            if result is not None:
+                self.log("Backlog resend unsuccessful for message: " + str(message["message"]["taskId"]), self.autosolve_constants.STATUS)
 
     ## VALIDATION FUNCTIONS ##
 
